import React from 'react'
import { Router, Route, Redirect, withRouter, Switch } from 'react-router-dom';
import { Provider } from 'react-redux';
import { store } from './components/redux/index';
import createHistory from "history/createBrowserHistory";

import './App.css';
// import logo from './logo.svg';

// импорт страниц
import { CUser } from './components/user';
import { CFeed } from './components/feed';
import { CComments } from './components/post';
import { CreatePost } from './components/create_post';
import AuthReg from './components/auth_reg';
// import Header from './components/structure/header';
import { CHeader } from './components/structure/header';
import Footer from './components/structure/footer';
import Search from './components/search';

// url проекта
export const url = 'http://hipstagram.node.ed.asmer.org.ua/graphql'

const history = createHistory()



{
    // фейковый авторизатор
    const fakeAuth = {
        isAuthenticated: false,
        authenticate(cb) {
            this.isAuthenticated = true
            setTimeout(cb, 100)
        },
        signout(cb) {
            this.isAuthenticated = false
            setTimeout(cb, 100)
        }
    }

    // кнопка разлогина, которая рисуется, если залогинен
    const AuthButton = withRouter(({ history }) => (
        fakeAuth.isAuthenticated && (
            <p>
                Welcome! <button onClick={() => {
                    fakeAuth.signout(() => history.push('/'))
                }}>Sign out</button>
            </p>
        )
    ))
}



const Page404 = () =>
    <div>
        NOT FOUND
    </div>



// Приватный роутинг
const PrivateRoute = ({ component: Component, ...rest }) => (
    <Route {...rest} render={(props) => (
        typeof localStorage.authToken === 'string'
            ? <Component {...props} />
            : <Redirect to={{
                pathname: '/authorization',
                state: { from: props.location }
            }} />
    )} />
)


const MyRoutes = () =>
    // <>
    <Switch>
        {/* <PrivateRoute path="/" component={CFeed} exact /> */}
        {/* <Route path="/" component={
            store.getState()?.auth?.token ? CFeed : Authorization
        } exact
        /> */}
        <Route path="/" component={
            store.getState()?.auth?.token ? CFeed : AuthReg
        } exact
        />

        {/* <Route path="/authorization" component={Authorization} /> */}
        {/* <Route path="/registration" component={Registration2} /> */}
        <Route path="/registration" component={AuthReg} />
        <Route path="/post/:postId" component={CComments} />
        <Route path="/user/:userId" component={CUser} />
        <Route path="/createpost" component={CreatePost} />
        <Route path='/search' component={Search} />
        <Route path="*" component={Page404} />
    </Switch>
{/* </> */ }


export default function App() {
    return (
        <Provider store={store}>
            <Router history={history}>
                <div className="wrapper">
                    <CHeader />
                    <main style={{ flexGrow: '1' }}>
                        <Switch>
                            <MyRoutes />
                        </Switch>
                    </main>
                    <Footer />
                </div>
            </Router>
        </Provider>

    )
}




// export default function App() {
//     return (
//         <Provider store={store}>
//             <Router history={history}>
//                 <div className="wrapper">
//                     <CHeader />
//                     <main style={{ flexGrow: '1' }}>
//                         <Switch>
//                             {/* <PrivateRoute path="/" component={CFeed} exact /> */}
//                             <Route path="/" component={
//                                 store.getState()?.auth?.token ? CFeed : Authorization
//                             } exact
//                             />

//                             <Route path="/authorization" component={Authorization} />
//                             <Route path="/registration" component={Registration} />
//                             <Route path="/post/:postId" component={CComments} />
//                             <Route path="/user/:userId" component={CUser} />
//                             <Route path="/createpost" component={CreatePost} />
//                             <Route path='/search' component={Search} />
//                             <Route path="*" component={Page404} />
//                         </Switch>
//                     </main>
//                     <Footer />
//                 </div>
//             </Router>
//         </Provider>

//     )
// }