import * as React from 'react';
import { store } from '../redux';
import { useState } from "react";
import { useHistory, useLocation } from "react-router-dom";
import { actionFullLogin } from "../redux/action";

import { Dialog, DialogActions, DialogContent, DialogContentText, DialogTitle, Paper, Box, Typography, Button, Stack, OutlinedInput, IconButton, InputLabel, InputAdornment, FormControl, TextField, Slide } from "@mui/material";

import { Visibility, VisibilityOff } from '@mui/icons-material';

// появление модального окна
const Transition = React.forwardRef(function Transition(props, ref) {
    return <Slide direction="up" ref={ref} {...props} />;
})


// поля ввода и кнопки
function LoginForm() {
    let location = useLocation();

    const history = useHistory();

    //  модальное окно 
    const [open, setOpen] = useState(false);
    const handleOpen = () => {
        setOpen(true);
    }
    const handleClose = () => {
        setOpen(false);
    }


    // отслеживаем введение данных в полях логин/пароль
    const [data, setData] = useState({ login: '', pass: '', confirmPass: '' })
    console.log('data: ', data)


    // проверка пароля на заданные параметры
    // регулярка, какие символы должны входить
    const pattern = /(?=.*[0-9])(?=.*[!@#$%^&*])(?=.*[a-z])(?=.*[A-Z])/g

    // минимальная длинна пароля
    const passLength = 8;

    let isValidate, isCorrect;

    ((data.pass.length < passLength) || !(pattern.test(data.pass))) && (isValidate = true);
    (data.pass != data.confirmPass) && (isCorrect = true);

    const validation = (data.login == '') || (data.pass == '')


    // включаем,выклюаем отображение пароля
    const [showPassword, setShowPassword] = useState(false);
    const handleClickShowPassword = () => setShowPassword((show) => !show);
    const handleMouseDownPassword = (event) => {
        event.preventDefault();
    };

    // включаем,выклюаем отображение подтверждения пароля
    const [showPasswordConfirm, setShowPasswordConfirm] = useState(false);
    const handleClickShowPasswordConfirm = () => setShowPasswordConfirm((show) => !show);
    const handleMouseDownPasswordConfirm = (event) => {
        event.preventDefault();
    };

    async function fullLogin() {
        const answer = await store.dispatch(actionFullLogin(data.login, data.pass))

        if (answer == null) {
            handleOpen()
        }

        if (store.getState()?.auth?.token) {
            history.push('/')
        }
    }

    return (
        <Box sx={{ width: '100%' }}>
            <form>
                <Stack spacing={2}>
                    <TextField
                        fullWidth
                        variant='outlined'
                        id='login'
                        label='Логин'
                        size='normal'
                        value={data.login}
                        onChange={e => setData({ ...data, login: e.target.value })}
                    />

                    <FormControl variant="outlined">
                        <InputLabel htmlFor="outlined-adornment-password">Пароль</InputLabel>
                        <OutlinedInput
                            error={isValidate}
                            fullWidth
                            id="password"
                            label="Пароль"
                            size="normal"
                            type={showPassword ? 'text' : 'password'}
                            value={data.pass}
                            onChange={e => setData({ ...data, pass: e.target.value })}

                            endAdornment={
                                <InputAdornment position="end">
                                    <IconButton
                                        aria-label="visibility"
                                        onClick={handleClickShowPassword}
                                        onMouseDown={handleMouseDownPassword}
                                        edge="end"
                                    >
                                        {showPassword ? <VisibilityOff /> : <Visibility />}
                                    </IconButton>
                                </InputAdornment>
                            }
                        />
                    </FormControl>

                    {(location.pathname === '/registration') && <FormControl variant="outlined">
                        <InputLabel htmlFor="outlined-adornment-password">Подтвердите пароль</InputLabel>
                        <OutlinedInput
                            error={isCorrect}
                            fullWidth
                            id="confirmPassword"
                            label="Подтвердите пароль"
                            size="normal"
                            type={showPasswordConfirm ? 'text' : 'password'}
                            value={data.confirmPass}
                            onChange={e => setData({ ...data, confirmPass: e.target.value })}

                            endAdornment={
                                <InputAdornment position="end">
                                    <IconButton
                                        aria-label="visibility"
                                        onClick={handleClickShowPasswordConfirm}
                                        onMouseDown={handleMouseDownPasswordConfirm}
                                        edge="end"
                                    >
                                        {showPasswordConfirm ? <VisibilityOff /> : <Visibility />}
                                    </IconButton>
                                </InputAdornment>
                            }
                        />
                    </FormControl>}

                    <Button
                        size="large"
                        variant="contained"
                        disabled={location.pathname === '/' ?
                            (validation || false) :
                            (validation || isCorrect) || false}
                        onClick={fullLogin}
                    >
                        {location.pathname === '/' ? 'Войти' : 'Регистрация'}
                    </Button>
                </Stack>
            </form>

            <div>
                <Dialog
                    open={open}
                    onClose={handleClose}
                    TransitionComponent={Transition}
                // keepMounted // всегда держит компонент смонтированным в DOM
                >
                    <DialogTitle>Что-то пошло не так!</DialogTitle>
                    <DialogContent>
                        <DialogContentText>
                            Вы ввели неправильный логин или пароль. Повторите попытку.
                        </DialogContentText>
                    </DialogContent>
                    <DialogActions>
                        <Button onClick={handleClose}>Ок</Button>
                    </DialogActions>
                </Dialog>
            </div>
        </Box>
    )
}


function RegistrationLink() {
    const history = useHistory()

    function goToRegistration() {
        history.push('/registration')
    }

    return (
        <Typography
            variant="subtitle1"
            sx={{
                display: 'grid',
                alignSelf: 'end',
                justifyContent: 'center',
                cursor: 'pointer'
            }}
            onClick={goToRegistration}
        >
            Зарегистрироваться
        </Typography>
    )
}


function AuthReg() {

    let location = useLocation();

    return (
        <Box sx={{
            display: 'grid',
            height: '80vh',
            justifyContent: 'center',
            alignItems: 'center'
        }}>

            <Paper elevation={3}
                sx={{
                    padding: '5px',
                    width: '400px',
                    height: '500px',
                    display: 'grid',
                    justifyContent: 'center',
                    alignItems: 'center'
                }}>
                <Typography
                    variant="h4"
                    noWrap
                    sx={{
                        display: 'grid',
                        fontFamily: 'monospace',
                        fontWeight: 700,
                        letterSpacing: '.3rem',
                        color: 'inherit',
                        textDecoration: 'none',
                        flexGrow: '1',
                        marginTop: '20px',
                        justifyContent: 'center'
                    }} >
                    Hipstagram
                </Typography>

                <Box
                    sx={{
                        display: 'grid',
                        alignSelf: 'start'
                    }}>
                    <LoginForm />
                </Box>

                {(location.pathname === '/') && <RegistrationLink />}
            </Paper>
        </Box >
    )
}

export default AuthReg